/*
 * Copyright by Inland Software
 *
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Intland Software. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Intland.
 */
package com.intland.codebeamer.wiki.plugins;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.math.NumberUtils;
import org.apache.log4j.Logger;
import org.apache.velocity.VelocityContext;

import com.ecyrd.jspwiki.WikiContext;
import com.ecyrd.jspwiki.plugin.PluginException;
import com.intland.codebeamer.datatransfer.MimeTargetWindow;
import com.intland.codebeamer.persistence.dao.impl.WikiPageCommentDaoImpl;
import com.intland.codebeamer.persistence.dto.UserDto;
import com.intland.codebeamer.persistence.dto.WikiPageCommentDto;
import com.intland.codebeamer.persistence.dto.WikiPageDto;
import com.intland.codebeamer.security.acl.Acl;
import com.intland.codebeamer.taglib.MimeIcon;
import com.intland.codebeamer.utils.FancyDate;
import com.intland.codebeamer.wiki.plugins.base.AbstractCodeBeamerWikiPlugin;

/**
 * A plugin that displays a list of recent page comments.
 *
 * @author <a href="mailto:kao@intland.com">Alexey Kolesnikov</a>
 * @version $Id$
 */
public class CommentsPlugin extends AbstractCodeBeamerWikiPlugin {
	private final static Logger log = Logger.getLogger(CommentsPlugin.class);

	private static final String PLUGIN_TEMPLATE = "comments-plugin.vm";

	private static final String PARAM_LIMIT = "limit";
	private static final String PARAM_ATTACHMENT = "attachment";

	// Not more that five comments by default.
	private Integer limit = new Integer(5);
	private Boolean showAttachments = Boolean.FALSE;
	private boolean limited = false;

	public String execute(WikiContext context, Map params) throws PluginException {
		String paramLimit = getParameter(params, PARAM_LIMIT);
		showAttachments = Boolean.valueOf(getParameter(params, PARAM_ATTACHMENT));

		if (NumberUtils.isNumber(paramLimit)) {
			if (log.isDebugEnabled()) {
				log.debug("Use limit: " + paramLimit);
			}
			limit = NumberUtils.createInteger(paramLimit);
		}

		UserDto user = getUserFromContext(context);
		WikiPageDto page = getPageFromContext(context);
		List<WikiPageCommentDto> comments = getPageComments(user, page);

		if (comments == null || comments.isEmpty()) {
			if (log.isDebugEnabled()) {
				log.debug("No page comments.");
			}
			return null;
		}

		List<Map<String,Object>> processed = processComments(context, comments);

		VelocityContext velocityContext = getDefaultVelocityContextFromContext(context);
		velocityContext.put("page", page);
		velocityContext.put("comments", processed);
		velocityContext.put("limited", Boolean.valueOf(limited));

		return renderPluginTemplate(PLUGIN_TEMPLATE, velocityContext);
	}

	/**
	 * Returns a list of comments processed to show in the template
	 */
	private List<Map<String,Object>> processComments(WikiContext context, List<WikiPageCommentDto> comments) {
		List<Map<String,Object>> result = new ArrayList<Map<String,Object>>(comments.size());
		UserDto user = getUserFromContext(context);

		for (WikiPageCommentDto dto : comments) {
			Map<String,Object> comment = new HashMap<String,Object>();
			result.add(comment);

			comment.put("id", dto.getId());
			comment.put("description", dto.getDescription());
			comment.put("submitter", Acl.findUser(dto.getSubmitter().getId()));
			comment.put("submittedAt", new FancyDate(dto.getSubmittedAt(), user, false));
			if (showAttachments.booleanValue() && dto.getFilename() != null) {
				comment.put("icon", MimeIcon.getIconName(dto.getMimeType()));
				comment.put("target", MimeTargetWindow.getTarget(context.getHttpRequest(), dto.getMimeType()));
				comment.put("filename", dto.getFilename());
				comment.put("size", dto.getDataLength());
			}
		}

		return result;
	}

	/**
	 * Returns the limited list of page comments.
	 */
	private List<WikiPageCommentDto> getPageComments(UserDto user, WikiPageDto page) {
		List<WikiPageCommentDto> result = WikiPageCommentDaoImpl.getInstance().findByWikiPageId(user, page.getId());
		if (result != null && result.size() > limit.intValue()) {
			result = result.subList(0, limit.intValue());
			limited = true;
		}
		return result;
	}
}
