/*
 * Copyright by Intland Software
 *
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Intland Software. ("Confidential Information"). You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Intland.
 */
package com.intland.codebeamer.wiki.plugins;

import java.io.File;
import java.io.InputStream;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import com.ecyrd.jspwiki.WikiContext;
import com.ecyrd.jspwiki.plugin.PluginException;
import com.intland.codebeamer.persistence.dao.ArtifactDao;
import com.intland.codebeamer.persistence.dao.impl.ArtifactDaoImpl;
import com.intland.codebeamer.persistence.dto.ArtifactDto;
import com.intland.codebeamer.persistence.dto.UserDto;
import com.intland.codebeamer.remoting.DescriptionFormat;
import com.intland.codebeamer.utils.Common;
import com.intland.codebeamer.wiki.CodeBeamerWikiContext;
import com.intland.codebeamer.wiki.WikiMarkupProcessor;
import com.intland.codebeamer.wiki.plugins.base.AbstractArtifactAwareWikiPlugin;

/**
 * Plugin to include wiki markup from an external source in a wiki page.
 * <p>
 * External sources can be either codeBeamer documents
 * (see the parameter <code>id</code>), or URLs (see the parameter <code>src</code>)
 * that point to wiki markup resources.
 *
 * @author <a href="mailto:zsolt.koppany@intland.com">Zsolt Koppany</a>
 * @version $Id:IncludePlugin.java 25316 2008-08-27 07:37:50Z zsolt $
 */
public class IncludePlugin extends AbstractArtifactAwareWikiPlugin {
	private static final Logger logger = Logger.getLogger(IncludePlugin.class);

	private final static String INCLUDED_DOC_IDS = IncludePlugin.class.getName() + "-included_doc_ids";
	private final static String SHOW_CONTROL = IncludePlugin.class.getName() + "-show-control";
	public final static String MANDATORY_PARAM_MESSAGE = "Either <tt>id</tt> or <tt>src</tt> must be provided";
	public final static String DOCUMENT_ID = "id";
	public final static String SRC = "src";

	private final WikiMarkupProcessor wikiMarkupProcessor = WikiMarkupProcessor.getInstance();
	private final ArtifactDao artifactDao = ArtifactDaoImpl.getInstance();

	private Integer pageId;
	private String srcUrl;

	public String execute(WikiContext wcontext, Map params) throws PluginException {
		String pageIdString = getParameter(params, DOCUMENT_ID);

		if (pageIdString != null) {
			try {
				pageId = Integer.valueOf(pageIdString);
			} catch (Throwable ex) {
				String msg = ex.toString();
				logger.warn(msg);
				return StringEscapeUtils.escapeHtml(msg);
			}
		}

		if (pageId == null) {
			srcUrl = getParameter(params, SRC);

			if (srcUrl != null) {
				try {
					File path = new File(wcontext.getEngine().getServletContext().getRealPath(srcUrl));
					if (path.canRead()) {
						srcUrl = path.getAbsolutePath();
					}
				} catch (Throwable ex) {
				}
			}
		}

		Object source = pageId == null ? srcUrl : pageId;
		if (source == null) {
			return MANDATORY_PARAM_MESSAGE;
		}

		logger.info("Source <" + source + ">");

		CodeBeamerWikiContext context = (CodeBeamerWikiContext)wcontext;

		HttpServletRequest request = context.getHttpRequest();

		boolean showControl = false;
		boolean removeControlAttributeFromRequest = false;

		try {
			showControl = ((Boolean)request.getAttribute(SHOW_CONTROL)).booleanValue();
		} catch (Throwable ex) {
			try {
				showControl = Boolean.parseBoolean(getParameter(params, "control", "false"));
			} catch (Throwable ex2) {
			}
			request.setAttribute(SHOW_CONTROL, Boolean.valueOf(showControl));
			removeControlAttributeFromRequest = true;
		}

		Set processedIncludes = (Set)request.getAttribute(INCLUDED_DOC_IDS);
		if (processedIncludes == null) {
			processedIncludes = new HashSet();
			request.setAttribute(INCLUDED_DOC_IDS, processedIncludes);
		} else if (processedIncludes.contains(source)) {
			logger.warn("Source <" + source + "> has been already processed");

			if (removeControlAttributeFromRequest) {
				request.removeAttribute(SHOW_CONTROL);
			}

			return "";
		}
		processedIncludes.add(source);

		StringBuffer html = new StringBuffer();
		if (showControl && pageId != null) {
			if (!writeControls(context.getHttpRequest().getContextPath(), context.getUser(),  pageId, html)) {
				if (removeControlAttributeFromRequest) {
					request.removeAttribute(SHOW_CONTROL);
				}
				return "";
			}
		}

		try {
			InputStream inputStream = getInputStream(context);
			String body = Common.readFileToString(inputStream, null);
			body = StringUtils.remove(body, (char)0);

			html.append(wikiMarkupProcessor.transformToHtml(context.getHttpRequest(), body, DescriptionFormat.WIKI));

			return html.toString();
		} catch (Throwable ex) {
			String msg = ex.toString();

			logger.warn(msg, ex);

			return StringEscapeUtils.escapeHtml(msg);
		} finally {
			if (removeControlAttributeFromRequest) {
				request.removeAttribute(SHOW_CONTROL);
			}
		}
	}

	protected boolean writeControls(String contextPath, UserDto user, Integer id, StringBuffer html) {
		ArtifactDto doc = artifactDao.findById(user, id);

		if (doc == null) {
			return false;
		}

		html.append("<a href=\"" + contextPath + doc.getUrlLink() + "\">" + StringEscapeUtils.escapeHtml(doc.getName()) + "</a>");
//		html.append(" <img src=\"" + contextPath +  doc.getIconUrl() + "\" />");
		html.append("<br />");

		return true;
	}

	/**
	 * Unused.
	 */
	@Override
	protected Integer getArtifactId() {
		return pageId;
	}

	/**
	 * Unused.
	 */
	@Override
	protected String getIconName() {
		return null;
	}

	/**
	 * Unused.
	 */
	@Override
	protected String getSrc() {
		return srcUrl;
	}
}
