package com.intland.codebeamer.wiki.plugins;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import com.ecyrd.jspwiki.WikiContext;
import com.ecyrd.jspwiki.plugin.PluginManager;
import com.intland.codebeamer.utils.Common;
import com.intland.codebeamer.wiki.CodeBeamerWikiContext;
import com.intland.codebeamer.wiki.plugins.base.AbstractArtifactAwareWikiPlugin;

/**
 * This plugin needs "latex" and "convert" (included on Redhat with ImageMagic) applications.
 *
 * Usage:
[{MathPlugin title='Optional title rendered as a tooltip on the image'

$math formula body$
}]

[{MathPlugin

\sqrt{1-e^2}
}]

[{MathPlugin

$\alpha$
}]

 *
 * @author <a href="calderoni@streamsim.com">Max Calderoni</a>
 * @author <a href="digiovinazzo@streamsim.com">Matteo Di Giovinazzo</a>
 * @author <a href="mailto:zsolt.koppany@intland.com">Zsolt Koppany</a>
 * @version $Id$
 */
public class MathPlugin extends AbstractArtifactAwareWikiPlugin {
	private final static Logger logger = Logger.getLogger(MathPlugin.class);

	private final static String binPath = File.separatorChar == '/' ? "/usr/bin/" : "";

	private Integer docId;
	private String src;

	public String execute(WikiContext wcontext, Map params) {
		// Processing parameters.
		try {
			docId = Integer.valueOf(getParameter(params, "id"));
		} catch (Exception ex) {
		}

		src = getParameter(params, "src");

		String expr = getParameter(params, "inline");
		if (StringUtils.isBlank(expr)) {
			expr = StringUtils.trimToEmpty((String)params.get(PluginManager.PARAM_BODY));
		}

		String title = getParameter(params, "title");
		String encoding = getParameter(params, "encoding", "UTF8");
		// Parameters processed.

		CodeBeamerWikiContext context = (CodeBeamerWikiContext)wcontext;

		try {
			if (docId != null) {
				// Using CodeBeamer document.
				InputStream inputStream = getInputStream(context);
				expr = Common.readFileToString(inputStream, encoding);
			}
			expr = StringUtils.remove(expr, (char)0);
			expr = StringUtils.trimToEmpty(expr);

			File image = File.createTempFile("cbmath", ".gif"); // FIXME change back to ".png" after FOP is able to handle transparent PNG pixelformat

			createImage(expr, image);

			logger.info("Created image file " + image.getAbsolutePath() + " length: " + image.length());

			String imgSrcPath = context.getHttpRequest().getContextPath() + "/writeChart/" + image.getName();

			// export img tag with appropriate src
			StringBuilder imgTag = new StringBuilder();
			imgTag.append("<img ");
			imgTag.append("src=\"" + imgSrcPath + "\" ");

			imgTag.append("alt=\"" + StringEscapeUtils.escapeHtml(expr) + "\" ");

			imgTag.append("title=\"" + StringEscapeUtils.escapeHtml(StringUtils.isBlank(title) ? expr : title) + "\" ");

			imgTag.append("style=\"vertical-align: middle;\" ");
			imgTag.append("/>");

			return imgTag.toString();
		} catch (Exception ex) {
			logger.warn(ex.getMessage(), ex);

			return StringEscapeUtils.escapeHtml(ex.getMessage());
		}
	}

	protected void createImage(final String expr, final File outputFile) throws Exception {
		outputFile.delete();

		File latexInputFile = File.createTempFile("cbmt", ".tex");
		latexInputFile.delete();

		PrintWriter writer = null;
		try {
			writer = new PrintWriter(new BufferedWriter(new FileWriter(latexInputFile)));

			//writer.println("\\documentclass[12pt]{article}");
			writer.println("\\documentclass{article}");
			writer.println("\\usepackage{amsmath}");
			writer.println("\\usepackage{amsfonts}");
			writer.println("\\usepackage{amssymb}");
			writer.println("\\pagestyle{empty}");

			// \DeclareMathSizes{10}{18}{12}{8}   % For size 10 text
			// \DeclareMathSizes{11}{19}{13}{9}   % For size 11 text
			// \DeclareMathSizes{12}{20}{14}{10}  % For size 12 text

			writer.println("\\begin{document}");
			writer.println(expr);
			writer.println("\\end{document}");
		} finally {
			if (writer != null) {
				writer.close();
			}
		}

		// The files below will be created by executed commands.
		String baseName = StringUtils.substringBeforeLast(latexInputFile.getName(), ".");
		File dviFile = new File(latexInputFile.getParent(), baseName + ".dvi");
		File psFile = new File(latexInputFile.getParent(), baseName + ".ps");

		try {
			// latex --interaction=nonstopmode math.tex
			List args = new ArrayList();
			args.add(binPath + "latex");
			args.add("--interaction=nonstopmode");
			args.add(latexInputFile.getAbsolutePath());

			execCommand(args, latexInputFile.getParentFile());

			// dvips

			args = new ArrayList();
			args.add(binPath + "dvips");
			args.add("-q");
			args.add("-E");
			args.add(dviFile.getAbsolutePath());
			args.add("-o");
			args.add(psFile.getAbsolutePath());

			execCommand(args, latexInputFile.getParentFile());

			// convert

			args = new ArrayList();
			args.add(binPath + "convert");
			args.add("-density");
			args.add("120");
// FIXME when FOP supports transparent PNG pixel format then these lines should be used...
//			args.add("-trim");
//			args.add("-transparent");
//			args.add("#FFFFFF");
// FIXME and these should be removed
args.add("-background");
args.add("white");
args.add("-flatten");
			args.add(psFile.getAbsolutePath());
			args.add(outputFile.getAbsolutePath());

			execCommand(args, latexInputFile.getParentFile());
		} finally {
			latexInputFile.delete();
			dviFile.delete();
			psFile.delete();

			File auxFile = new File(latexInputFile.getParent(), baseName + ".aux");
			auxFile.delete();

			File logFile = new File(latexInputFile.getParent(), baseName + ".log");
			logFile.delete();
		}
	}

	protected void execCommand(List args, File workingDir) throws Exception {
		String[] command = (String[])args.toArray(new String[]{});

		logger.info("Exec: \"" + StringUtils.join(command, "\" \"") + "\"");

		Process process = Runtime.getRuntime().exec(command, null, workingDir);
		int exitCode = process.waitFor();
		if (exitCode != 0) {
			logger.info("Exit: " + exitCode);
		}
	}

	public Integer getArtifactId() {
		return docId;
	}

	public String getSrc() {
		return src;
	}

	public String getIconName() {
		return null;
	}
}
