/*
 * Copyright by Intland Software
 *
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Intland Software. ("Confidential Information"). You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Intland.
 */
package com.intland.codebeamer.wiki.plugins;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.log4j.Logger;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.tools.generic.RenderTool;

import com.ecyrd.jspwiki.WikiContext;
import com.intland.codebeamer.manager.TrackerManager;
import com.intland.codebeamer.manager.WorkingSetManager;
import com.intland.codebeamer.persistence.dao.TrackerItemDao;
import com.intland.codebeamer.persistence.dto.ProjectDto;
import com.intland.codebeamer.persistence.dto.UserDto;
import com.intland.codebeamer.persistence.dto.UserTrackerItemStatsDto;
import com.intland.codebeamer.persistence.dto.UserTrackerItemsGroupedByProjectDto;
import com.intland.codebeamer.persistence.dto.WorkingSetDto;
import com.intland.codebeamer.servlet.bugs.BrowseTrackerAction;
import com.intland.codebeamer.taglib.actionmenu.model.ActionItem;
import com.intland.codebeamer.ui.view.actionmenubuilder.ProjectListPageActionMenuBuilder;
import com.intland.codebeamer.wiki.plugins.base.AbstractCodeBeamerWikiPlugin;

/**
 * Plugin to display/select open items on all projects by different criterias.
 *
 * Any necessary selectors/filters can be added/removed in java part without
 * modifying the template.
 *
 * @author <a href="mailto:kao@intland.com">Alexey Kolesnikov</a>
 * @version $Id$
 */
public class MyOpenTrackerItemsPlugin extends AbstractCodeBeamerWikiPlugin {
	private final static Logger logger = Logger.getLogger(MyOpenTrackerItemsPlugin.class);

	public static final String ITEMS_FILTER = "itemsFilter";
	public static final String ONLY_DIRECT_USER_ITEMS = "onlyDirectUserItems";

	private static final String LAYOUT_MATRIX = "myopentrackeritems-matrix-plugin.vm";
	private static final String LAYOUT_SELECT = "myopentrackeritems-plugin.vm";

	/** List of selectors to be displayed. */
	private List<Selection> selectors = Collections.emptyList();
	/** List of filters applied to each selector. */
	private List<Selection> filters = makeFilters();
	private List<ProjectEntry> projectsList = new ArrayList<ProjectEntry>();

	private WorkingSetManager workingSetManager = WorkingSetManager.getInstance();
	private TrackerManager trackerManager = TrackerManager.getInstance();

	public String execute(WikiContext context, Map params) {
		long startTime = System.currentTimeMillis();

		// get beans from wiki context
		UserDto user = getUserFromContext(context);
		WorkingSetDto workingSet = null;
		if (workingSetManager.getConfiguration(user) == WorkingSetManager.CONFIGURATION_WORKINGSET) {
			List<WorkingSetDto> workingSets = workingSetManager.findByUser(user, true);
			if(workingSets != null && !workingSets.isEmpty()) {
				workingSet = workingSets.get(0);
			}
		}

		// parse params
		String layoutType = getParameter(params, "layout", "");
		String template = (layoutType.equalsIgnoreCase("select")) ? LAYOUT_SELECT : LAYOUT_MATRIX;

		boolean onlyDirectUserItems = "true".equalsIgnoreCase(getParameter(params, ONLY_DIRECT_USER_ITEMS, "false"));
		selectors = makeSelectors(onlyDirectUserItems);

		// Insert an entry for the all-projects in workingset
		UserTrackerItemStatsDto total = new UserTrackerItemStatsDto();
		ProjectEntry allProjects = new ProjectEntry(workingSet, null, total);
		allProjects.setProjectName("All Projects");
		projectsList.add(allProjects);

		// Get all projects in current user working set
		Set<ProjectDto> workingSetProjects = new HashSet<ProjectDto>(workingSetManager.getSelectedWorkingsetProjects(user));

		long timeToGetWorkingSets = System.currentTimeMillis();
		if (logger.isDebugEnabled()) {
			logger.debug("Took " + (timeToGetWorkingSets - startTime) + " ms. to get working-sets, user: " + user);
		}

		// Get the tracker issue statistics for the user
		List<UserTrackerItemsGroupedByProjectDto> stats = trackerManager.getTrackerItemStatsByUserGroupedByProjects(user, null, workingSetProjects, false, onlyDirectUserItems);
		for (UserTrackerItemsGroupedByProjectDto proj : stats) {
			// Only show projects' statistics in current working set
			if (workingSetProjects.contains(proj.getProject())) {
				projectsList.add(new ProjectEntry(null, proj.getProject(), proj));
				total.add(proj);
			}
		}

		long timeToGetStatistics = System.currentTimeMillis();
		if (logger.isDebugEnabled()) {
			logger.debug("Took " + (timeToGetStatistics - timeToGetWorkingSets) + " ms. to get statistics, user: " + user);
		}

		// set up Velocity context
		VelocityContext velocityContext = getDefaultVelocityContextFromContext(context);
		velocityContext.put("defaultProjectSelection", Integer.valueOf(0));  //'All Projects' selection by default
		velocityContext.put("userId", user.getId());
		velocityContext.put("projectsList", projectsList);
		velocityContext.put("numProjects", Integer.valueOf(projectsList.size() -1));
		velocityContext.put("selectors", selectors);
		velocityContext.put("filters", filters);
		velocityContext.put("contextPath", getApplicationContextPath(context));
		velocityContext.put("onlyDirectUserItems", Boolean.valueOf(onlyDirectUserItems));

		putAll(velocityContext, buildSelectedWorkingSetVars(context, user));

		// Necessary only for Velocity 1.5 and older
		velocityContext.put("ctx", velocityContext);
		velocityContext.put("render", new RenderTool());

		// render template
		String html = renderPluginTemplate(template, velocityContext);

		long timeToRender = System.currentTimeMillis();
		if (logger.isDebugEnabled()) {
			logger.debug("Took " + (timeToRender - timeToGetStatistics) + " ms. to render, user: " + user);
		}

		return html;
	}

	/**
	 * Put the selected working-set and its link as url to context.
	 * @param context The current wiki context
	 * @param user The current user
	 *
	 * @return The variables for the current selected working-set
	 */
	private Map buildSelectedWorkingSetVars(WikiContext context, UserDto user) {
		ProjectListPageActionMenuBuilder projectListPageActionMenuBuilder = (ProjectListPageActionMenuBuilder) getApplicationContext(context).getBean("projectListPageActionMenuBuilder");

		ActionItem action = projectListPageActionMenuBuilder.buildWorkingSetAction(context.getHttpRequest(), user);

		Map vars = new HashMap();
		if (action != null) {
			vars.put("selectedWorkingSetName", action.getLabel());
			vars.put("selectedWorkingSetURL" , action.getUrl());
		}
		return vars;
	}

	/**
	 * Parameters storage class for both selectors and filters
	 */
	public class Selection {
		/** Title to be displayed in form */
		private String name;

		/**
		 * prefix field is used in Velocity template to combine all the
		 * properties stated in UserTrackerItemStatsDto
		 */
		private String prefix;

		/** Selection parameters as key-vaule pairs */
		private Map<String,Object> parameters = new HashMap<String,Object>(8);

		public String getName() {
			return name;
		}

		Selection(String name, String prefix) {
			this.name = name;
			this.prefix = prefix;
		}

		public String getPrefix() {
			return prefix;
		}

		public void setPrefix(String prefix) {
			this.prefix = prefix;
		}

		public Set<Map.Entry<String,Object>> getParametersSet() {
			return parameters.entrySet();
		}

		public void addParametersPair(String key, Object value) {
			this.parameters.put(key, value);
		}

		public String toString() {
			return (new ToStringBuilder(this)).toString();
		}
	}

	public class ProjectEntry {
		private WorkingSetDto workingSet;
		private ProjectDto project = null;
		private String projectName;
		private UserTrackerItemStatsDto stats;

		ProjectEntry(WorkingSetDto workingSet, ProjectDto project, UserTrackerItemStatsDto stats) {
			setWorkingSet(workingSet);
			setProject(project);
			setStats(stats);
		}

		public WorkingSetDto getWorkingSet() {
			return workingSet;
		}

		public void setWorkingSet(WorkingSetDto workingSet) {
			this.workingSet = workingSet;
		}

		public ProjectDto getProject() {
			return project;
		}

		public void setProject(ProjectDto project) {
			this.project = project;
			projectName = project != null ? project.getName() : "";
		}

		public String getProjectName() {
			return projectName;
		}

		public void setProjectName(String projectName) {
			this.projectName = projectName;
		}

		public UserTrackerItemStatsDto getStats() {
			return stats;
		}

		public void setStats(UserTrackerItemStatsDto stats) {
			this.stats = stats;
		}

		public int getAll() {
			return stats != null ? stats.getAll() : 0;
		}

		public String toString() {
			return "MyOpentTrackerItemsPlugin$ProjectEntry[project=" + projectName +", stats=" + stats +"]";
		}

	}

	/**
	 * Prepare selectors.
	 */
	private List<Selection> makeSelectors(boolean onlyDirectUserItems) {
		List<Selection> result = new ArrayList<Selection>();
		Selection sel;

		sel = new Selection("All", "all");
		sel.addParametersPair(BrowseTrackerAction.ONLY_ASSIGNED_TO_USER, Boolean.TRUE);
		sel.addParametersPair(BrowseTrackerAction.ONLY_SUBMITTED_BY_USER, Boolean.TRUE);
		sel.addParametersPair(BrowseTrackerAction.ONLY_OWNED_BY_USER, Boolean.TRUE);
		sel.addParametersPair(ONLY_DIRECT_USER_ITEMS, Boolean.valueOf(onlyDirectUserItems));
		result.add(sel);

		sel = new Selection("Assigned to me", "assigned");
		sel.addParametersPair(BrowseTrackerAction.ONLY_ASSIGNED_TO_USER, Boolean.TRUE);
		sel.addParametersPair(ONLY_DIRECT_USER_ITEMS, Boolean.valueOf(onlyDirectUserItems));
		result.add(sel);

		sel = new Selection("Submitted by me", "submitted");
		sel.addParametersPair(BrowseTrackerAction.ONLY_SUBMITTED_BY_USER, Boolean.TRUE);
		result.add(sel);

		sel = new Selection("Owned by me", "supervised");
		sel.addParametersPair(BrowseTrackerAction.ONLY_OWNED_BY_USER, Boolean.TRUE);
		sel.addParametersPair(ONLY_DIRECT_USER_ITEMS, Boolean.valueOf(onlyDirectUserItems));
		result.add(sel);

		return result;
	}

	/**
	 * Prepare filters.
	 */
	private List<Selection> makeFilters() {
		List<Selection> result = new ArrayList<Selection>();
		Selection sel;

		sel = new Selection("All", "");
		sel.addParametersPair(ITEMS_FILTER, Integer.valueOf(TrackerItemDao.TF_NOFILTER));
		result.add(sel);

		sel = new Selection("Overdue", "Overdue");
		sel.addParametersPair(ITEMS_FILTER, Integer.valueOf(TrackerItemDao.TF_OVERDUE));
		result.add(sel);

		sel = new Selection("Today Overdue", "TodayAndOverdue");
		sel.addParametersPair(ITEMS_FILTER, Integer.valueOf(TrackerItemDao.TF_TODAY_AND_OVERDUE));
		result.add(sel);

		sel = new Selection("Today", "Today");
		sel.addParametersPair(ITEMS_FILTER, Integer.valueOf(TrackerItemDao.TF_TODAY));
		result.add(sel);

		sel = new Selection("Tomorrow", "Tomorrow");
		sel.addParametersPair(ITEMS_FILTER, Integer.valueOf(TrackerItemDao.TF_TOMORROW));
		result.add(sel);

		sel = new Selection("Next 7 Days Overdue", "Next7DaysAndOverdue");
		sel.addParametersPair(ITEMS_FILTER, Integer.valueOf(TrackerItemDao.TF_NEXT_7_DAYS_AND_OVERDUE));
		result.add(sel);

		sel = new Selection("Next 7 Days", "Next7Days");
		sel.addParametersPair(ITEMS_FILTER, Integer.valueOf(TrackerItemDao.TF_NEXT_7_DAYS));
		result.add(sel);

		sel = new Selection("This Month", "CurrentMonth");
		sel.addParametersPair(ITEMS_FILTER, Integer.valueOf(TrackerItemDao.TF_CURRENT_MONTH));
		result.add(sel);

		return result;
	}
}
