/*
 * Copyright by Intland Software
 *
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Intland Software. ("Confidential Information"). You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Intland.
 */
package com.intland.codebeamer.wiki.plugins;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.math.NumberUtils;
import org.apache.velocity.VelocityContext;

import com.ecyrd.jspwiki.WikiContext;
import com.intland.codebeamer.persistence.dao.ArtifactCommentDao;
import com.intland.codebeamer.persistence.dao.ArtifactDao;
import com.intland.codebeamer.persistence.dao.impl.ArtifactCommentDaoImpl;
import com.intland.codebeamer.persistence.dao.impl.ArtifactDaoImpl;
import com.intland.codebeamer.persistence.dto.ArtifactDto;
import com.intland.codebeamer.persistence.dto.ProjectDto;
import com.intland.codebeamer.persistence.dto.UserDto;
import com.intland.codebeamer.persistence.dto.base.AbstractCommentDto;
import com.intland.codebeamer.remoting.ArtifactType;
import com.intland.codebeamer.wiki.plugins.base.AbstractCodeBeamerWikiPlugin;
import com.intland.codebeamer.wiki.plugins.base.NamedPluginException;

/**
 * Plugin to display list of recently changed/commented wiki pages.
 *
 * @author <a href="mailto:kao@intland.com">Alexey Kolesnikov</a>
 * @version $Id$
 */
public class RecentWikiPagesPlugin extends AbstractCodeBeamerWikiPlugin {
	private static final String RENDER_TEMPLATE = "recentwikipages-plugin.vm";
	private static final Integer WIKI_PAGE_TYPE = Integer.valueOf(ArtifactType.PROJECT_WIKIPAGE);

	private final static ArtifactDao artifactDao = ArtifactDaoImpl.getInstance();
	private final static ArtifactCommentDao artifactCommentDao = ArtifactCommentDaoImpl.getInstance();

	public String execute(WikiContext context, Map params) {
		UserDto user = getUserFromContext(context);
		ProjectDto project;
		try {
			project = discoverProject(params, context);
		} catch (NamedPluginException ex) {
			return renderErrorTemplate(ex);
		}

		int max = NumberUtils.toInt(getParameter(params, "max"), 5);
		int since = NumberUtils.toInt(getParameter(params, "since"), 100);

		Collection wikiUpdates = getUpdates(user, project, max, since);

		// set up Velocity context
		VelocityContext velocityContext = getDefaultVelocityContextFromContext(context);
		velocityContext.put("wikiUpdates", wikiUpdates);
		velocityContext.put("isCurrentProject", isEnclosingProject());
		velocityContext.put("project", project);
		velocityContext.put("contextPath", getApplicationContextPath(context));

		// render template
		return renderPluginTemplate(RENDER_TEMPLATE, velocityContext);
	}

	protected Collection<AbstractCommentDto> getUpdates(UserDto user, ProjectDto project, int max, int since) {
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DATE, -1 * since);

		List<AbstractCommentDto> wikiUpdates = Collections.emptyList();

		List<ArtifactDto> recentPages = artifactDao.findRecentlyModifiedArtifacts(user, project, WIKI_PAGE_TYPE, cal.getTime(), max);
		if (recentPages != null && !recentPages.isEmpty()) {
			wikiUpdates = new ArrayList<AbstractCommentDto>(recentPages.size());
			wikiUpdates.addAll(artifactDao.findHistoryEntriesByArtifactAndDate(user, recentPages));
			wikiUpdates.addAll(artifactCommentDao.findByArtifactAndDate(user, recentPages));

			// merge into a sorted list by using a comparator that sorts entries by reversed date order
			Collections.sort(wikiUpdates, new Comparator<AbstractCommentDto>() {
				public int compare(AbstractCommentDto obj1, AbstractCommentDto obj2) {
					return obj2.getSubmittedAt().compareTo(obj1.getSubmittedAt());
				}
			});

			// trim list to the right length
			if (wikiUpdates.size() > max) {
				wikiUpdates = wikiUpdates.subList(0, max);
			}
		}

		return wikiUpdates;
	}
}
