/*
 * Copyright by Intland Software
 *
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Intland Software. ("Confidential Information"). You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Intland.
 */
package com.intland.codebeamer.wiki.plugins.base;

import java.util.Map;

import org.apache.velocity.VelocityContext;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.validation.BindException;
import org.springframework.validation.DataBinder;
import org.springframework.validation.ObjectError;

import com.ecyrd.jspwiki.plugin.PluginException;

/**
 * Simple Wiki-plugin base class uses Spring data-binding to bind WIKI plugin parameters to a command object.
 * A bit modelled after the Spring's Abstract/BaseCommandController, but not exactly.
 *
 * @param <Command> Generics parameter, the type for the command bean.
 *
 * @author <a href="mailto:zoltan.luspai@intland.com">Zoltan Luspai</a>
 * $Id$
 */
public abstract class AbstractCommandWikiPlugin<Command> extends AutoWiringCodeBeamerPlugin {
	/** Name for the command during binding. */
	protected String commandName = "";

	/**
	 * Populate the velocity context, before the template gets rendered.
	 * @param velocityContext
	 * @param params
	 *
	 * @throws PluginException
	 */
	public final void populateContext(VelocityContext velocityContext, Map params) throws PluginException {
		Command command = createCommand();
		DataBinder binder = createBinder(command);
		initBinder(binder, command, params);
		binder.bind(new MutablePropertyValues(params));
		onBind(binder, command, params);
		if (binder.getBindingResult().hasErrors()) {
			onBindingErrors(binder, command, params);
		} else {
			validate(binder, command, params);
		}
		if (binder.getBindingResult().hasErrors()) {
			throw new PluginException(binder.getBindingResult().toString(), new BindException(binder.getBindingResult()));
		}
		// binding and validation successful, populate the model
		Map model = populateModel(binder, command, params);
		if (model != null) {
			putAll(velocityContext, model);
		}
	}

	/**
	 * Helper to add some error message about an property
	 * @param binder The binder
	 * @param objectName The object name
	 * @param msg And the error message
	 */
	protected void addError(DataBinder binder, String objectName, String msg) {
		binder.getBindingResult().addError(new ObjectError(objectName, msg));
	}

	/**
	 * Create a new command where the parameters will be bound to.
	 * @return
	 *
	 * @throws PluginException
	 */
	protected abstract Command createCommand() throws PluginException;

	/**
	 * Create the Databinder for the command.
	 * @param command
	 * @return
	 */
	protected DataBinder createBinder(Command command) {
		DataBinder binder = new DataBinder(command, commandName);
		return binder;
	}

	/**
	 * Override this method to initialize the binder.
	 * @param binder
	 * @param command
	 * @param params
	 */
	protected void initBinder(DataBinder binder, Command command, Map params) {
	}

	/**
	 * Custom post-processing after the binder finished allows custom binding.
	 * @param binder
	 * @param command
	 * @param params
	 */
	protected void onBind(DataBinder binder, Command command, Map params) {
	}

	/**
	 * Called after the binding done, if the binding has errors. Override to handle them
	 * @param binder
	 * @param command
	 * @param params
	 */
	protected void onBindingErrors(DataBinder binder, Command command, Map params) {
	}

	/**
	 * Validate called when binding is ok, to validate the bound object.
	 * @param binder
	 * @param command
	 * @param params
	 */
	protected void validate(DataBinder binder, Command command, Map params){
	}

	/**
	 * Populate the model to be used with Velocity context
	 * @param binder The databinder used.
	 * @param command The command object bound.
	 * @param params The map contains WIKI parameters
	 * @return The model object
	 *
	 *  @throws PluginException
	 */
	protected abstract Map populateModel(DataBinder binder, Command command, Map params) throws PluginException;

}
