/*
 * Copyright by Intland Software
 *
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Intland Software. ("Confidential Information"). You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Intland.
 */
package com.intland.codebeamer.wiki.plugins.dataset.producer;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Map;

import org.apache.log4j.Logger;
import org.jfree.data.general.Dataset;
import org.jfree.data.time.Millisecond;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;

import com.ecyrd.jspwiki.plugin.PluginException;
import com.intland.codebeamer.persistence.util.PersistenceUtils;
import com.intland.codebeamer.servlet.CBPaths;
import com.intland.codebeamer.wiki.plugins.dataset.WikiDataSetProducer;

/**
 * @author <a href="mailto:zsolt.koppany@intland.com">Zsolt Koppany</a>
 * @version $Id$
 */
public class CodeBeamerResourceTrendProducer extends AbstractWikiDataSetProducer implements WikiDataSetProducer {
	private static Logger logger = Logger.getLogger(CodeBeamerResourceTrendProducer.class);

	private final static String LOG_DATE_FOMAT = "yyyy-MM-dd HH:mm:ss,SSS";
	private final static int KB_TO_MB = 1024;

	public Dataset produce(Map<String, String> params) throws PluginException  {
		if (getParams() == null) {
			setParams(params);
		}

		try {
			boolean showUsed = Boolean.parseBoolean(params.get("showUsed"));

			long startTime = System.currentTimeMillis();

			Dataset data = getWikiDataSet(showUsed);

			long endTime = System.currentTimeMillis();

			logger.info("Elapsed: " + NumberFormat.getInstance(Locale.US).format(endTime - startTime) + " ms.");
			return data;
		} catch (IOException ex) {
			throw new PluginException(ex);
		}
	}

	protected Dataset getWikiDataSet(boolean showUsed) throws IOException, PluginException {
		BufferedReader reader = new BufferedReader(new InputStreamReader(getInputStream()));

		try {
			SimpleDateFormat dateFormat = new SimpleDateFormat(LOG_DATE_FOMAT, Locale.ENGLISH);
			NumberFormat numberFormat = NumberFormat.getInstance(Locale.US);

			Date startDate = getStartDate();
			if (startDate == null) {
				startDate = PersistenceUtils.getToday(-3);
			}
			Date endDate = getEndDate();

			TimeSeries freeTimeSeries = new TimeSeries("Free", Millisecond.class);
			TimeSeries totalTimeSeries = new TimeSeries("Total", Millisecond.class);
			TimeSeries maxTimeSeries = new TimeSeries("Max", Millisecond.class);
			TimeSeries rssTimeSeries = new TimeSeries("Rss", Millisecond.class);
			TimeSeries usedTimeSeries = new TimeSeries("Used", Millisecond.class);
			TimeSeries threadTimeSeries = new TimeSeries("Threads", Millisecond.class);

			boolean isRssAvailable = false;
			for (String line = null, previousTimeStamp = ""; (line = reader.readLine()) != null; ) {
				try {
					if (logger.isTraceEnabled()) {
						logger.trace("Line <" + line + ">");
					}

					//2008-11-05 04:32:13,862 - Memory [5.2.2] - free: 43,389 kb (57%) total: 75,776 kb max: 256,384 kb rss: 124,316 kb threads: 15 runable: 3 waiting: 1/11 blocked: 0 new: 0 terminated: 0

					int off = line.indexOf(" - ");
					if (off == -1) {
						logger.warn("Unparseable line <" + line + ">");
						continue;
					}

					String timeStamp = line.substring(0, off);
					if (timeStamp.equals(previousTimeStamp)) {
						// Entries with the same time-stamp must be skipped because charts might have problems
						// with them.
						continue;
					}

					previousTimeStamp = timeStamp;

					Date date = null;
					try {
						date = dateFormat.parse(timeStamp);
					} catch (ParseException ex) {
						logger.warn("Couldn't parse date <" + timeStamp + "> in line <" + line + ">");
						continue;
					}

					Millisecond millisecond = new Millisecond(date);

					if (startDate != null && date.before(startDate)) {
						continue;
					}

					if (endDate != null && date.after(endDate)) {
						continue;
					}

					line = line.substring(off + 1).trim();

					off = line.indexOf(':');
					line = line.substring(off + 1).trim();
					//43,389 kb (57%) total: 75,776 kb max: 256,384 kb rss: 124,316 kb threads: 15 runable: 3 waiting: 1/11 blocked: 0 new: 0 terminated: 0
					off = line.indexOf(' ');
					Number free = parseNumber(numberFormat, line.substring(0, off), KB_TO_MB);

					off = line.indexOf(':');
					line = line.substring(off + 1).trim();
					//75,776 kb max: 256,384 kb rss: 124,316 kb threads: 15 runable: 3 waiting: 1/11 blocked: 0 new: 0 terminated: 0
					off = line.indexOf(' ');
					Number total = parseNumber(numberFormat, line.substring(0, off), KB_TO_MB);

					off = line.indexOf(':');
					line = line.substring(off + 1).trim();
					//256,384 kb rss: 124,316 kb threads: 15 runable: 3 waiting: 1/11 blocked: 0 new: 0 terminated: 0
					off = line.indexOf(' ');
					Number max = parseNumber(numberFormat, line.substring(0, off), KB_TO_MB);

					off = line.indexOf("rss:");
					isRssAvailable = off != -1;
					if (isRssAvailable) {
						off = line.indexOf(':');
						line = line.substring(off + 1).trim();
						//124,316 kb threads: 15 runable: 3 waiting: 1/11 blocked: 0 new: 0 terminated: 0
						off = line.indexOf(' ');
						Number rss = parseNumber(numberFormat, line.substring(0, off), KB_TO_MB);

						rssTimeSeries.add(millisecond, rss);
					}

					off = line.indexOf(':');
					line = line.substring(off + 1).trim();
					//15 runable: 3 waiting: 1/11 blocked: 0 new: 0 terminated: 0
					off = line.indexOf(' ');
					Number threads = parseNumber(numberFormat, line.substring(0, off), 1);

					freeTimeSeries.add(millisecond, free);
					totalTimeSeries.add(millisecond, total);
					maxTimeSeries.add(millisecond, max);
					usedTimeSeries.add(millisecond, new Long(total.longValue() - free.longValue()));
					threadTimeSeries.add(millisecond, threads);
				} catch (ParseException ex) {
					logger.warn(ex + " in line <" + line + "> ");
				} catch (StringIndexOutOfBoundsException ex) {
					logger.warn(ex + " in line <" + line + "> ", ex);
				}
			}

			// The colors: 1. red, 2. blue, 3. green, 4. yellow, 5. magenta, 6. cyan
			TimeSeriesCollection timeseriesCollection = new TimeSeriesCollection();

			if (!showUsed) {
				timeseriesCollection.addSeries(freeTimeSeries);
				timeseriesCollection.addSeries(totalTimeSeries);
				timeseriesCollection.addSeries(maxTimeSeries);
				if (isRssAvailable) {
					timeseriesCollection.addSeries(rssTimeSeries);
				}
				timeseriesCollection.addSeries(threadTimeSeries);
			} else {
				timeseriesCollection.addSeries(usedTimeSeries);
				timeseriesCollection.addSeries(freeTimeSeries);
				timeseriesCollection.addSeries(totalTimeSeries);
				timeseriesCollection.addSeries(maxTimeSeries);
				if (isRssAvailable) {
					timeseriesCollection.addSeries(rssTimeSeries);
				}
				timeseriesCollection.addSeries(threadTimeSeries);
			}

			return timeseriesCollection;
		} finally {
			reader.close();
		}
	}

	protected Number parseNumber(NumberFormat numberFormat, String pattern, int divider) throws ParseException {
		Number num = numberFormat.parse(pattern);
		if (num != null && divider != 1) {
			num = new Long(num.longValue() / divider);
		}

		return num;
	}

	protected InputStream getInputStream() throws IOException {
		File tomcatDir = CBPaths.getInstance().getTomcatDir();
		File memoryFile = new File(tomcatDir, "logs/memory.txt");

		return new FileInputStream(memoryFile);
	}
}
