/*
 GoogleMaps Plugin

 Copyright (C) 2006 i3G Institut - Hochschule Heilbronn

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.intland.codebeamer.wiki.plugins.googlemaps;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.regex.Matcher;

import com.ecyrd.jspwiki.WikiContext;
import com.ecyrd.jspwiki.plugin.PluginException;
import com.intland.codebeamer.persistence.dto.GroupDto;
import com.intland.codebeamer.persistence.dto.ProjectDto;
import com.intland.codebeamer.persistence.dto.UserDto;
import com.intland.codebeamer.security.acl.Acl;
import com.intland.codebeamer.wiki.plugins.base.AbstractCodeBeamerWikiPlugin;
import com.intland.codebeamer.wiki.plugins.base.NamedPluginException;

/**
 * The GoogleMaps plugin inserts a Map on the Wiki page and allows the user to
 * place markers and lines on the map. Markers can display an info window when
 * moving the mouse over them.
 *
 * @author Amir Farokhzad
 */
public class GoogleMaps extends AbstractCodeBeamerWikiPlugin {

	public static final String KEY_PROPERTY = "jspwiki.Google.maps.key";

	/** Marker start. Example: "Red Marker P1: 100,200" */
	public static final String REGEX_MARKER = ".*Marker.*:.*";

	public static final String REGEX_LINES = "(.*)Lines:.*";

	private static final String KEY_NOT_FOUND = "The {{GoogleMaps}} plugin __requires__ to "
			+ "[obtain an API key from Google Maps|http://www.google.com/apis/maps/]. \\\\"
			+ "Add the API key as a new property {{jspwiki.Google.maps.key=...}} "
			+ "to the {{WEB-INF/classes/" + PropertyManager.PROPERTY_FILE +"}} configuration file.";

	/** The complete HTML and JavaScript code for the map */
	private String map;

	private WikiContext wikiContext;

	/**
	 * Creates the JavaScript code which is needed to show the plugin on the
	 * jspwiki.
	 *
	 * @throws PluginException
	 */
	public String execute(WikiContext context, Map params) throws PluginException {
		this.wikiContext = context;
		try {
			PropertyManager.init();
		} catch (Exception ex) {
			return ex.getMessage();
		}
		String lat = (String) params.get("lat");
		String lng = (String) params.get("lng");
		String zoom = (String) params.get("zoom");
		String mtype = (String) params.get("type");
		String mcontrol = (String) params.get("control");
		String key = (String) params.get("key");
		String width = (String) params.get("width");
		String height = (String) params.get("height");
		String trace = (String) params.get("trace");
		String fixIEStr = (String) params.get("fixIE");
		boolean fixIE = false;
		if (fixIEStr != null && fixIEStr.equalsIgnoreCase("yes"))
			fixIE = true;
		// Always will use safe js code
		fixIE = true;

		if (key == null || key.equals("")) {
			key = PropertyManager.getProperty(KEY_PROPERTY);
			if (key == null || key.equals("")) {
				return context.getEngine().textToHTML(context, KEY_NOT_FOUND);
			}
		}

		// CodeBeamer-specific parameters
		// FIXME: implement as a separate module
		StringBuffer markerLines = new StringBuffer("");
		StringBuffer membersInfo = new StringBuffer ("");
		boolean showMembers = "true".equalsIgnoreCase((String) params.get("showMembers"));
		if (showMembers) {
			ProjectDto project;
			try {
				project = discoverProject(params, context);
			} catch (NamedPluginException ex) {
				return renderErrorTemplate(ex);
			}
			// Get project members list
			GroupDto group = Acl.findGroup(project.getGroupId());
			Collection members = Acl.getGroupUsersRoles(group, true, null, null);
			membersInfo.append("Total-" + members.size());

			int locatedMembers = 0;
			for (Iterator it = members.iterator(); it.hasNext();) {
				Map member = (Map)it.next();
				UserDto user = (UserDto)member.get("user");

				StringBuffer address = new StringBuffer("");
				StringBuffer countryZip = new StringBuffer("");
				if (user.getCountry() != null &&  user.getCountry().length() > 0 ) {
					countryZip.append(user.getCountry());
				}

				if (user.getCity() != null && user.getCity().length() > 0) {
					if (user.getAddress() != null && user.getAddress().length() > 0) {
						address.append(user.getAddress() + ",");
					}
					address.append(user.getCity() + ",");
				}
				address.append(countryZip.toString());

				String query = "http://maps.google.com/maps/geo?q=" + address.toString().replaceAll(" ", "+") + "&output=csv&key=" + key;
				// Get coordinates
				StringBuffer result = new StringBuffer("");

				try {
					URL url = new URL(query);
					BufferedReader content = new BufferedReader(new InputStreamReader(url.openStream()));
					String line = "";
					while ((line = content.readLine()) != null) {
						result.append(line);
					}
					content.close();
				} catch (IOException e) {
					continue;
				}

				// Parce csv responce, like "200,6,42.730070,-73.690570"
				// Address accuracy codes info: http://www.google.com/apis/maps/documentation/reference.html#GGeoAddressAccuracy
				String latLng = null;
				String[] fields = result.toString().split(",");
				if (!fields[0].startsWith("2")) {
					// Non-successful http result
					continue;
				}
				try {
					int accuracy = Integer.parseInt(fields[1].trim());
					if (accuracy == 0) {
						// Unknown location
						continue;
					}
					double latData = Double.parseDouble(fields[2].trim());
					double lngData = Double.parseDouble(fields[3].trim());
					latLng = "" + latData + ", " + lngData;
				} catch (Exception e) {
					continue;
				}

				markerLines.append("\nMarker: " + latLng + "\n" + user.getName()+ " (" + address.toString() + ")\n\n");
				locatedMembers++;
			}

			if (locatedMembers == 0) {
				return "No project members with valid addresses found.";
			}
			membersInfo.append(" Located-" + locatedMembers);
		}
		// End of CodeBeamer-specific parameters

		if (lat == null || lat.equals("")) {
			lat = "49.12271669366686";
		}

		if (lng == null || lng.equals("")) {
			lng = "9.206113815307617";
		}

		if (zoom == null || zoom.equals("")) {
			zoom = "5";
		}

		if (mtype == null) {
			mtype = "G_NORMAL_MAP";
		} else if (mtype.trim().equalsIgnoreCase("map")) {
			mtype = "G_NORMAL_MAP";
		} else if (mtype.trim().equalsIgnoreCase("sat")) {
			mtype = "G_SATELLITE_MAP";
		} else if (mtype.trim().equalsIgnoreCase("hybrid")) {
			mtype = "G_HYBRID_TYPE";
		} else {
			mtype = "G_NORMAL_MAP";
		}
		if (mcontrol == null) {
			mcontrol = "new GLargeMapControl()";
		} else if (mcontrol.trim().equalsIgnoreCase("small")) {
			mcontrol = "new GSmallMapControl()";
		} else {
			mcontrol = "new GLargeMapControl()";
		}

		if (width == null || width.equals("")) {
			width = "400";
		}

		if (height == null || height.equals("")) {
			height = "400";
		}

		String body = (String) params.get("_body");
		if (body == null)
			body = "";
		else {
			body = body.replaceAll("\r\n", "\n");
			body = body.replaceAll("\r", "\n");
		}
		// CB-specific: add generated markers to body lines
		body += markerLines.toString();
		// end of CB-specific
		String[] bodyLines = body.trim().split("\n");

		int widthInt = Integer.parseInt(width);

		map = "<!-- Start of GoogleMaps Code -->\n";
		map += "<script src=\"http://maps.google.com/maps?file=api&v=2&key=" + key + "\" type=\"text/javascript\"></script>\n";
		map += "<div id=\"map\" style=\"width: " + width + "px; height: " + height + "px\"></div>\n";

		if (trace != null && trace.trim().toLowerCase().equals("true")) {
			map += "<div id=\"coordinate\">\n";
			map += "<div style=\"width: "
					+ widthInt
					/ 3
					+ "px; padding-left:1px; padding-top:3px; padding-bottom:3px; background-color:#FFFF00; float:left;\">Latitude: <span id=\"lat\">"
					+ lat + "</span></div>\n";
			map += "<div style=\"width: "
					+ widthInt
					/ 3
					+ "px; padding-top:3px; padding-bottom:3px; background-color:#FFFF00; float:left;\">Longitude: <span id=\"lng\">"
					+ lng + "</span></div>\n";
			map += "<div style=\"width: " + widthInt / 6
					+ "px; padding-top:3px; padding-bottom:3px; background-color:#FFFF00; float:left;\">Zoom: <span id=\"zoom\">"
					+ zoom + "</span></div>\n";
			map += "<div style=\"width: "
					+ widthInt
					/ 6
					+ "px; padding-top:3px; padding-bottom:3px; background-color:#FFFF00; float:left;\"><a href=\"http://www.i3g.hs-heilbronn.de\">Plugin by i3G</a></div>\n";
			map += "</div>\n";
			if (showMembers) {
				map += "<br/><div>\n";
				map += "<strong>Members: </strong>" + membersInfo + "\n";
				map += "</div>\n";
			}
		}

		map += "<script type=\"text/javascript\">\n";
		map += "//<![CDATA[\n";

		if (fixIE)
			map += "function googlemaps (){\n";

		// CB-specific:
		map += "var markerBounds = new GLatLngBounds();\n";
		// end of CB-specific
		map += "var map = new GMap2(document.getElementById(\"map\"));\n";
		map += "map.addControl(" + mcontrol + ");\n";
		map += "map.addControl(new GScaleControl());\n";
		map += "map.addControl(new GMapTypeControl());\n";
		map += "var latlng = new GLatLng(" + lat + ", " + lng + ");\n";
		map += "map.setCenter(latlng, " + zoom + ", " + mtype + ");\n";

		Marker[] markers = findMarkers(bodyLines);
		handleJavascriptObjects(markers);

		Polyline[] polyline = GoogleMaps.findpolyLines(bodyLines);
		handleJavascriptObjects(polyline);

		// FIX: error in original code.
		// The DOM elements on which the listeners try to get access
		// will be present in code if only "trace=true" is specified
		if (trace != null && trace.trim().toLowerCase().equals("true")) {
			addInfoListener();
		}
		// CB-specific: sep pan/zoom to get all markers visible
		if (showMembers) {
			map += "map.setCenter(markerBounds.getCenter(), map.getBoundsZoomLevel(markerBounds), " + mtype + ");\n";
		}
		// end of CB-specific
		map += "map.addControl(new GOverviewMapControl(new GSize(150,150)));\n";

		if (fixIE) {
			map += "}\n";
			// CB-specific: safe call of function when it's available only.
			// Necessary for IE7
			map += "YAHOO.util.Event.onAvailable(\"map\", googlemaps);\n";
			// end of CB-specific
		}

		map += "//]]>\n";
		map += "</script>\n";
		map += "<!-- End of GoogleMaps Code -->\n";
		// Clear marker list
		Marker.getMarkerList().clear();
		IconManager.clear();
		return map;

	}

	/**
	 * Searches for marker entries in the plugin body. These are defined in a
	 * marker section, wich startes with a line containing "Marker:". For every
	 * marker line a new marker object is created.
	 *
	 * @param bodyLines
	 *            The lines of the plugin body
	 * @return An array containing the created marker objects
	 */
	private Marker[] findMarkers(String[] bodyLines) {
		boolean markerSection = false;
		ArrayList markerList = new ArrayList();
		String markerCode = "";
		for (int i = 0; i < bodyLines.length; i++) {
			String bodyLine = bodyLines[i].trim();

			// Marker starts here
			// TODO RegEx
			Matcher matcher = RegEx.getMatcher(REGEX_MARKER, bodyLine);
			if (matcher.matches()) {
				markerSection = true;
			}

			if (markerSection) {
				markerCode += bodyLines[i] + "\n";
				String nextLine;
				if (i < bodyLines.length - 1) {
					nextLine = bodyLines[i + 1];
				} else {
					nextLine = "";
				}

				// Marker section ends when an empty line appears
				// and no "Tab:" is found after the empty line
				// or when the end of the body is reached
				// Now create Marker
				boolean moreTabs = nextLine.startsWith("Tab:");
				if ((bodyLines[i].length() == 0 && !moreTabs) || i == bodyLines.length - 1) {
					Marker marker = new Marker(markerCode.trim(), wikiContext);
					markerList.add(marker);
					markerCode = "";
					markerSection = false;
				}
			}
		}

		int length = markerList.size();
		return (Marker[]) markerList.toArray(new Marker[length]);

	}

	/**
	 *
	 * @param markers
	 */
	private void handleJavascriptObjects(JavascriptObject[] objects) {
		for (int i = 0; i < objects.length; i++) {
			map += objects[i].createJavaScript();
		}
	}

	public static Polyline[] findpolyLines(String[] bodyLines) {
		boolean lineSection = false;
		ArrayList pLineList = new ArrayList();
		String color = "";
		for (int i = 0; i < bodyLines.length; i++) {
			String bodyLine = bodyLines[i].trim();
			if (bodyLine.length() == 0)
				continue;
			Matcher matcher = RegEx.getMatcher(REGEX_LINES, bodyLine);
			if (matcher.matches()) {
				lineSection = true;
				color = matcher.group(1).trim().toLowerCase();
			} else if (bodyLine.indexOf("Marker") >= 0) {
				lineSection = false;
			} else if (lineSection) {
				Polyline pLine = new Polyline(bodyLine, color);
				pLineList.add(pLine);
			}
		}
		int length = pLineList.size();
		return (Polyline[]) pLineList.toArray(new Polyline[length]);

	}

	public void addInfoListener() {
		map += "GEvent.addListener(map, 'moveend', function() {\n";
		map += "  document.getElementById(\"lat\").innerHTML = map.getCenter().y\n";
		map += "  document.getElementById(\"lng\").innerHTML = map.getCenter().x\n";
		map += "  document.getElementById(\"zoom\").innerHTML = map.getZoom()\n";
		map += "});\n";
	}
}
